﻿namespace Phoenix.Main.Frame
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using Phoenix.Log.Writer;
    using Phoenix.Main.Configuration;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.View.Sessions;
    using Phoenix.Translation;
    using Phoenix.View;
    using Phoenix.View.Helpers;
    using Phoenix.View.Management;
    using Phoenix.View.Search;

    using WeifenLuo.WinFormsUI.Docking;

    /// <summary>
    /// Main window and entry point of Phoenix Suite
    /// </summary>
    public partial class PhoenixMain : TranslatedForm
    {
        #region Fields

        private readonly LogFrame logFrame = new LogFrame();
        private readonly List<PatientManagerBox> patientManagerBoxes = new List<PatientManagerBox>();
        private readonly SideMenu sideMenu;

        private AppearanceConfiguration appearanceConfig = new AppearanceConfiguration();
        private GeneralConfiguration generalConfig = new GeneralConfiguration();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Phoenix"/> class.
        /// </summary>
        public PhoenixMain()
            : base()
        {
            InitializeComponent();

            this.Logger = LogManager.GetLogger(typeof(PhoenixMain));
            sideMenu = new SideMenu(this.dockPanel);

            this.sideMenu.Hidden += sideMenu_Hidden;
            this.sideMenu.SearchPatientClicked += tsmi_OpenSession_Click;
        }

        #endregion Constructors

        #region Methods

        private PatientManagerBox CreateNewPatientManagerBox(Patient patient)
        {
            PatientManagerBox.Configuration configuration = new PatientManagerBox.Configuration()
            {
                CriterionPanelHeight = appearanceConfig.PatientCriterionPanelHeight,
                FrameAction = FrameAction.Insert,
                SearchBoxView = appearanceConfig.SearchBoxView
            };
            PatientManagerBox fr;
            fr = new PatientManagerBox(configuration);
            this.patientManagerBoxes.Add(fr);
            return fr;
        }

        private void DEBUG_Adddoctor(object sender, EventArgs e)
        {
            DoctorManagerBox fr = new DoctorManagerBox();
            fr.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_AddDoctortype(object sender, EventArgs e)
        {
            DoctorTypeManagerBox fr = new DoctorTypeManagerBox();
            fr.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_AddFamilyLink(object sender, EventArgs e)
        {
            PatientSearchBox search = new PatientSearchBox();
            DialogResult dr = search.ShowDialog();
            if (dr != DialogResult.OK) return;

            FamilyLinkDisplayBox box = new FamilyLinkDisplayBox((Patient)search.SelectedItem);
            box.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_AddFamilyLinkType(object sender, EventArgs e)
        {
            FamilyLinkTypeManagerBox fr = new FamilyLinkTypeManagerBox(FrameAction.Insert);
            fr.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_AddInsurance_Click(object sender, EventArgs e)
        {
            InsuranceManagerBox fr = new InsuranceManagerBox();
            fr.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_AddPractice(object sender, EventArgs e)
        {
            PracticeManagerBox fr = new PracticeManagerBox();
            fr.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_AddReputation(object sender, EventArgs e)
        {
            ReputationManagerBox fr = new ReputationManagerBox();
            fr.ShowDialog();
            this.sideMenu.RefreshManagers();
        }

        private void DEBUG_Captcha(object sender, EventArgs e)
        {
            CaptchaBox fr = new CaptchaBox();
            fr.ConfirmationFailed += delegate(object sender1, EventArgs e1)
            {
                MessageBox.Show("Captcha failed!");
            };
            fr.ConfirmationSucceeded += delegate(object sender2, EventArgs e2)
            {
                MessageBox.Show("Captcha succeeded!");
            };
            fr.ShowDialog();
        }

        private void DEBUG_RefreshManagers(object sender, EventArgs e)
        {
            this.sideMenu.RefreshManagers();
        }

        private void fr_ConfirmationFailed(object sender, EventArgs e)
        {
            throw new NotImplementedException();
        }

        private void NotYetImplemented()
        {
            MessageBoxes.Information("This feature is not yet implemented!");
        }

        private void PhoenixMain_Load(object sender, EventArgs e)
        {
            try
            {
                this.sideMenu.ShowHint = DockState.DockLeft;
                this.sideMenu.Show(this.dockPanel);
                this.Logger.Debug("SideMenu configured and loaded");

                this.logFrame.ShowHint = DockState.DockBottom;
                this.logFrame.Show(this.dockPanel);
                this.Logger.Debug("LogFrame configured and loaded");
            #if DEBUG
                this.tsmi_Debug.Visible = true;
                this.tsmi_LogFrame.Checked = true;
                this.logFrame.Show();
            #else
            this.tsmi_Debug.Visible = false;
            this.logFrame.Hide();
            #endif
                this.RefreshShortcuts();
            }
            catch (Exception ex)
            {
                this.Logger.Error(ex);
                Application.Exit();
            }
        }

        private void RefreshShortcuts()
        {
            try
            {
                ShortcutConfiguration shortcuts = new ShortcutConfiguration();
                this.tsmi_CloseSession.ShortcutKeys = shortcuts.Session_CloseSession;
                this.tsmi_OpenSession.ShortcutKeys = shortcuts.Session_OpenSession;
                this.tsmi_Configuration.ShortcutKeys = shortcuts.File_Configuration;
                this.tsmi_Quit.ShortcutKeys = shortcuts.File_Quit;
                this.tsmi_QuickInsertPatient.ShortcutKeys = shortcuts.Session_QuickInsertPatient;
                this.tsmi_InsertPatient.ShortcutKeys = shortcuts.Session_InsertPatient;
                this.tsmi_NavigationMenu.ShortcutKeys = shortcuts.View_NavigationMenu;
                this.tsmi_LogFrame.ShortcutKeys = shortcuts.View_Log;
                this.Logger.Debug("Shortcuts refreshed.");
            }
            catch (Exception ex)
            {
                base.Logger.Error(ex);
            #if DEBUG
                MessageBoxes.Error("Impossible to load the shortcuts: " + ex.Message);
            #else
                MessageBoxes.Error("Impossible to load the shortcuts. Usually it is because you setup shortcuts with only 'shift + key' or just a key.");
            #endif
            }
        }

        private void sideMenu_Hidden(object sender, EventArgs e)
        {
            this.tsmi_NavigationMenu.Checked = !this.tsmi_NavigationMenu.Checked;
        }

        private void tsmi_About_Click(object sender, EventArgs e)
        {
            AboutBox fr = new AboutBox();
            fr.ShowDialog();
        }

        private void tsmi_CloseSession_Click(object sender, EventArgs e)
        {
            SessionDisposerBox fr = new SessionDisposerBox(this.sideMenu.SessionsCollection);
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.Cancel) return;

            foreach (Session session in fr.ClosedSessionsPatients)
            {
                session.Save();
                session.Close();
            }
            this.sideMenu.RefreshSessions();
        }

        private void tsmi_Configuration_Click(object sender, EventArgs e)
        {
            ConfigurationBox fr = new ConfigurationBox();
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
                this.RefreshShortcuts();
        }

        private void tsmi_InsertPatient_Click(object sender, EventArgs e)
        {
            PatientManagerBox.Configuration configuration = new PatientManagerBox.Configuration()
            {
                CriterionPanelHeight = appearanceConfig.DoctorCriterionPanelHeight,
                SearchBoxView = appearanceConfig.SearchBoxView,
                FrameAction = FrameAction.Insert,
                MobileNumberDisplayPattern = generalConfig.MobileNumberDisplayPattern,
                PhoneNumberDisplayPattern = generalConfig.PhoneNumberDisplayPattern,
                MinDate = appearanceConfig.MinDate
            };
            PatientManagerBox fr = new PatientManagerBox(configuration);
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;

            if (fr.ToBeDisplayed)
                this.sideMenu.Add(new Session(fr.ItemToManage));
        }

        private void tsmi_LogFrame_Click(object sender, EventArgs e)
        {
            if (this.logFrame == null)
            {
                this.Logger.Warning("Impossible to show the LogFrame because it is null!");
                return;
            }

            this.tsmi_LogFrame.Checked = !this.tsmi_LogFrame.Checked;
            if (tsmi_LogFrame.Checked)
                this.logFrame.Show();
            else
                this.logFrame.Hide();
        }

        private void tsmi_NavigationMenu_Click(object sender, EventArgs e)
        {
            if (tsmi_NavigationMenu.Checked)
            {
                this.sideMenu.Hide();
                this.tsmi_NavigationMenu.Checked = false;
            }
            else
            {
                this.sideMenu.Show();
                this.tsmi_NavigationMenu.Checked = true;
            }
        }

        private void tsmi_OpenSession_Click(object sender, EventArgs e)
        {
            PatientSearchBox fr = new PatientSearchBox(appearanceConfig.PatientCriterionPanelHeight
                                                       , appearanceConfig.SearchBoxView
                                                       , appearanceConfig.MinDate);
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.Cancel) return;

            Patient patient = fr.SelectedItem as Patient;
            this.sideMenu.Add(new Session(patient));
        }

        private void tsmi_QuickInsertPatient_Click(object sender, EventArgs e)
        {
            PatientQuickInsertBox fr = new PatientQuickInsertBox();
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;

            if (fr.ToBeDisplayed)
                this.sideMenu.Add(new Session(fr.ItemToManage));
        }

        private void tsmi_SaveAllAndClose_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBoxes.Question("Do you want to quit Phoenix?".Translate(), MessageBoxButtons.YesNo);
            if (dr == DialogResult.Yes)
                this.Close();
        }

        #endregion Methods
    }
}