﻿namespace Phoenix.Model.Core.Management
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter.Interfaces;

    /// <summary>
    /// Manage the data of the patient
    /// </summary>
    public class PatientDisplayerModel
    {
        #region Fields

        private IInsuranceRepository insuranceRepository = RepositoryFactory.Get<IInsuranceRepository>();
        private List<Insurance> insurances = new List<Insurance>();
        private IPatientRepository patientRepository = RepositoryFactory.Get<IPatientRepository>();
        private IPracticeRepository practiceRepository = RepositoryFactory.Get<IPracticeRepository>();
        private List<Practice> practices = new List<Practice>();
        private IPatientManagerPresenter presenter;
        private IReputationRepository reputationRepository = RepositoryFactory.Get<IReputationRepository>();
        private List<Reputation> reputations = new List<Reputation>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientDisplayerModel"/> class.
        /// </summary>
        /// <param name="presenter">The presenter.</param>
        public PatientDisplayerModel(IPatientManagerPresenter presenter)
        {
            this.presenter = presenter;
            this.presenter.Refreshed += (sender, e) => this.Refresh();
            this.presenter.Updated += (sender, e) => this.Update();
            this.presenter.PatientLoaded += (sender, e) =>
            {
                var patient = this.presenter.Patient;
                if (patient == null) return;

                patient.Counter++;
                this.patientRepository.Update(patient);
            };
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Determines whether this instance can add the specified doctor.
        /// </summary>
        /// <param name="doctor">The doctor.</param>
        /// <returns>
        /// 	<c>true</c> if this instance [can add doctor] the specified doctor; otherwise, <c>false</c>.
        /// </returns>
        public bool CanAddDoctor(Doctor doctor)
        {
            return (from doc in this.presenter.Patient.Doctors
                    where doc.ID == doctor.ID
                    select doc).Count() == 0;
        }

        private void Refresh()
        {
            using (var thread = new BackgroundWorker())
            {
                this.presenter.BackgroundWork = true;
                this.reputations = this.reputationRepository.GetAllRows(OrderBy.ASC);
                this.insurances = this.insuranceRepository.GetAllRows(OrderBy.ASC);
                this.practices = this.practiceRepository.GetAllRows(OrderBy.ASC);

                this.presenter.Reputations = this.reputations;
                this.presenter.Insurances = this.insurances;
                this.presenter.Practices = this.practices;

                this.presenter.RefreshGui();
                this.presenter.BackgroundWork = false;
            }
        }

        private void Update()
        {
            this.presenter.Patient.LastName = this.presenter.Patient.LastName.ToUpper();
            this.patientRepository.Update(this.presenter.Patient);
        }

        #endregion Methods
    }
}