﻿namespace Phoenix.Model.Management
{
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    /// <summary>
    /// Manage drugs
    /// </summary>
    public class PathologyModel : Manager<Pathology>
    {
        #region Fields

        private IPathologyInputPresenter presenter;
        private IPathologyTypeRepository ptRepository = RepositoryFactory.Get<IPathologyTypeRepository>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PathologyModel"/> class.
        /// </summary>
        /// <param name="presenter">The view.</param>
        public PathologyModel(IPathologyInputPresenter presenter)
            : base(RepositoryFactory.Get<IPathologyRepository>())
        {
            this.presenter = presenter;
            this.presenter.Inserted += (sender, e) => this.Repository.Create(this.presenter.ItemToManage);
            this.presenter.Updated += (sender, e) => this.Repository.Update(this.presenter.ItemToManage);
            this.presenter.Deleted += (sender, e) => this.Repository.Delete(this.presenter.ItemToManage);
            this.presenter.Refreshed += (sender, e) => this.RefreshData();
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c>
        /// </returns>
        public override bool ValidateData()
        {
            this.presenter.ClearError();
            bool isValid = true;
            if (string.IsNullOrEmpty(this.presenter.ItemToManage.Title))
            {
                this.presenter.SetError(PathologyFailedOn.NoName, "The pathology doesn't have a name".Translate());
                isValid = false;
            }
            if (this.presenter.ItemToManage.Type == null)
            {
                this.presenter.SetError(PathologyFailedOn.NoType, "The pathology doesn't have a type".Translate());
                isValid = false;
            }
            return isValid;
        }

        /// <summary>
        /// Process to execute when a delete is raised.
        /// </summary>
        /// <param name="item">The item to delete.</param>
        protected override void Delete(Pathology item)
        {
            if (this.Repository.IsReferenced(this.presenter.ItemToManage))
            {
                this.presenter.SetError(PathologyFailedOn.Referenced, "Impossible to delete this item, it is referenced somewhere else.");
            }
            else
            {
                base.Delete(item);
            }
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        private void RefreshData()
        {
            this.presenter.Types = this.ptRepository.GetAllRows();
        }

        #endregion Methods
    }
}