﻿namespace Phoenix.Presenter.Sessions
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using Phoenix.Contracts;

    /// <summary>
    /// Represents a collection of session gui managers.
    /// </summary>
    public class SessionCollection : IEnumerable<Session>
    {
        #region Fields

        private List<Session> sessionList = new List<Session>();

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets the number of elements actually contained in the SessionCollection.
        /// </summary>
        /// <value>The number of elements actually contained in the SessionCollection.</value>
        public int Count
        {
            get { return this.sessionList.Count; }
        }

        #endregion Properties

        #region Indexers

        /// <summary>
        /// Gets or sets the <see cref="Phoenix.Model.Sessions.Session"/> with the specified idx.
        /// </summary>
        /// <value></value>
        public Session this[int idx]
        {
            get { return this.sessionList[idx]; }
            set { this.sessionList[idx] = value; }
        }

        #endregion Indexers

        #region Methods

        /// <summary>
        /// Performs an implicit conversion from <see cref="Phoenix.View.Sessions.SessionCollection"/> 
        /// to <see cref="System.Collections.Generic.List&lt;Phoenix.View.Sessions.Session&gt;"/>.
        /// </summary>
        /// <param name="collection">The collection.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator List<Session>(SessionCollection collection)
        {
            return collection.sessionList;
        }

        /// <summary>
        /// Performs an explicit conversion from <see cref="System.Collections.Generic.List&lt;Phoenix.View.Sessions.Session&gt;"/> 
        /// to <see cref="Phoenix.View.Sessions.SessionCollection"/>.
        /// </summary>
        /// <param name="sessions">The sessions.</param>
        /// <returns>The result of the conversion.</returns>
        public static explicit operator SessionCollection(List<Session> sessions)
        {
            SessionCollection c = new SessionCollection();
            c.AddRange(sessions);
            return c;
        }

        /// <summary>
        /// Add a new session in the collection.
        /// </summary>
        /// <param name="session">The session.</param>
        public void Add(Session session)
        {
            if (!this.sessionList.Exists(current => session.Patient.ID == current.Patient.ID))
            {
                this.sessionList.Add(session);
                return;
            }
            throw new ArgumentException(string.Format("The session already exist for {0}!", session.Patient));
        }

        /// <summary>
        /// Adds a list sessions into the collection.
        /// </summary>
        /// <param name="sessions">The sessions.</param>
        public void AddRange(List<Session> sessions)
        {
            this.sessionList.AddRange(sessions);
        }

        /// <summary>
        /// Removes all elements SessionCollection.
        /// </summary>
        public void Clear()
        {
            this.sessionList.Clear();
        }

        /// <summary>
        /// Closes all sessions.
        /// </summary>
        public void CloseAll()
        {
            foreach (var session in this.sessionList) session.Close();
        }

        /// <summary>
        /// Returns <c>True</c> if the specified Session exists in one of the SessionGui of the list.
        /// Otherwise returne <c>False</c>
        /// </summary>
        /// <param name="session">The session.</param>
        /// <returns>
        /// Returns <c>True</c> if the specified SessionGui exists in the list.
        /// Otherwise returne <c>False</c>d
        /// </returns>
        public bool Contains(Session session)
        {
            return this.sessionList.Exists(current => current.Patient.ID == session.Patient.ID);
        }

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1"/> that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<Session> GetEnumerator()
        {
            foreach (Session item in sessionList) yield return item;
        }

        /// <summary>
        /// Refreshes all the sessions
        /// </summary>
        public void Refresh()
        {
            foreach (Session session in this.sessionList)
            {
                session.RefreshData();
            }
        }

        /// <summary>
        /// Removes all the ISessionGui that contains the specified session and close all the opend managers
        /// </summary>
        /// <param name="session">The session.</param>
        /// <param name="save">if set to <c>true</c> save the session before closing it. Otherwise close without saving.</param>
        /// <returns>
        /// The number of SessionGui removed from the list.
        /// </returns>       
        public int Remove(Session session, bool save)
        {
            Assert.IsTrue((session != null), "The session to remove is null!");

            Session currentSession = this.sessionList.FirstOrDefault(current => session.Patient == current.Patient);
            if (currentSession != null)
            {
                currentSession.Close();
            }

            int count = this.sessionList.RemoveAll(current => current.Patient.ID == session.Patient.ID);

            Assert.IsTrue((count >= 1), string.Format("{0} sessions have been deleted! Zero or one was expected.", count));
            return count;
        }

        /// <summary>
        /// Removes all the ISessionGui that contains the specified session.
        /// This call WILL save the session before closing it.
        /// </summary>
        /// <param name="session">The session.</param>
        /// <returns>The number of SessionGui removed from the list.</returns>        
        public int Remove(Session session)
        {
            return this.Remove(session, true);
        }

        /// <summary>
        /// Removes all the closed sessions.
        /// </summary>
        public void RemoveClosedSessions()
        {
            List<Session> toRemove = new List<Session>();
            foreach (Session session in this.sessionList)
            {
                if (session.IsClosed)
                {
                    toRemove.Add(session);
                }
            }

            foreach (Session session in toRemove)
            {
                this.sessionList.Remove(session);
            }
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion Methods
    }
}