﻿namespace Phoenix.View.Frame
{
    using System;
    using System.Management;
    using System.Reflection;
    using System.Threading;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Helpers.Misc;
    using Phoenix.Translation;
    using Phoenix.View.Frame.Controls;

    /// <summary>
    /// Contains the info about Phoenix Suite.
    /// </summary>
    partial class AboutBox : TranslateableForm
    {
        #region Fields

        public const string SOFTWARE = "nDoctor 2";

        private const int INTERVAL = 2500;

        private System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();

        #endregion Fields

        #region Constructors

        public AboutBox()
        {
            InitializeComponent();
            this.TranslateForm();

            base.Logger = LogManager.GetLogger(typeof(AboutBox));

            this.labelProductName.Text = SOFTWARE;
            this.labelVersion.Text = String.Format("Version {0}", this.AssemblyVersion);
            this.labelCopyright.Text = this.AssemblyCopyright;
            this.labelCompanyName.Text = "Written by: {0}".Translate(this.AssemblyCompany);
            this.textBoxDescription.Text = this.GetLicense();
            this.labelDate.Text = "Built the {0} between {1} and {2}.".Translate(this.AssemblyReleaseDate.ToShortDateString()
             , this.AssemblyReleaseDate.ToShortTimeString()
             , this.AssemblyReleaseDate.AddHours(3).ToShortTimeString());

            this.FillComponentVersion();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the assembly company.
        /// </summary>
        /// <value>The assembly company.</value>
        public string AssemblyCompany
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCompanyAttribute), false);
                if (attributes.Length == 0)
                {
                    return string.Empty;
                }
                return ((AssemblyCompanyAttribute)attributes[0]).Company;
            }
        }

        /// <summary>
        /// Gets the assembly copyright.
        /// </summary>
        /// <value>The assembly copyright.</value>
        public string AssemblyCopyright
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
                if (attributes.Length == 0)
                {
                    return string.Empty;
                }
                return ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
            }
        }

        /// <summary>
        /// Gets the assembly description.
        /// </summary>
        /// <value>The assembly description.</value>
        public string AssemblyDescription
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyDescriptionAttribute), false);
                if (attributes.Length == 0)
                {
                    return string.Empty;
                }
                return ((AssemblyDescriptionAttribute)attributes[0]).Description;
            }
        }

        /// <summary>
        /// Gets the assembly product.
        /// </summary>
        /// <value>The assembly product.</value>
        public string AssemblyProduct
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyProductAttribute), false);
                if (attributes.Length == 0)
                {
                    return string.Empty;
                }
                return ((AssemblyProductAttribute)attributes[0]).Product;
            }
        }

        /// <summary>
        /// Gets the assembly date.
        /// </summary>
        /// <value>The assembly date.</value>
        public DateTime AssemblyReleaseDate
        {
            get
            {
                return Revision.CalculateRevisionDate(Assembly.GetExecutingAssembly().GetName().Version.Revision);
            }
        }

        /// <summary>
        /// Gets the assembly title.
        /// </summary>
        /// <value>The assembly title.</value>
        public string AssemblyTitle
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
                if (attributes.Length > 0)
                {
                    AssemblyTitleAttribute titleAttribute = (AssemblyTitleAttribute)attributes[0];
                    if (string.IsNullOrEmpty(titleAttribute.Title))
                    {
                        return titleAttribute.Title;
                    }
                }
                return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
            }
        }

        /// <summary>
        /// Gets the assembly version.
        /// </summary>
        /// <value>The assembly version.</value>
        public string AssemblyVersion
        {
            get
            {
                return Assembly.GetExecutingAssembly().GetName().Version.ToString();
            }
        }

        #endregion Properties

        #region Methods

        public uint CPUSpeed()
        {
            ManagementObject Mo = new ManagementObject("Win32_Processor.DeviceID='CPU0'");
            uint sp = (uint)(Mo["CurrentClockSpeed"]);
            Mo.Dispose();
            return sp;
        }

        private void AboutBox_Load(object sender, EventArgs e)
        {
            this.Text = String.Format("About {0}", SOFTWARE);
            string url = "http://www.phoenixsuite.net";

            CreditsViewer.TeamMember tm = new CreditsViewer.TeamMember();
            tm.FirstName = "Jean-Baptiste";
            tm.LastName = "Wautier";
            tm.Function = "Project Owner";
            tm.Website = url;
            tm.Picture = ResourceHelper.GetImage(typeof(AboutBox), "Resources.Jibe.JPG");
            this.cv_Credits.Add(tm);

            tm = new CreditsViewer.TeamMember();
            tm.FirstName = "Valérian";
            tm.LastName = "Lecoq";
            tm.Function = "Tester".Translate();
            tm.Website = url;
            tm.Picture = ResourceHelper.GetImage(typeof(AboutBox), "Resources.Val.jpg");
            this.cv_Credits.Add(tm);

            tm = new CreditsViewer.TeamMember();
            tm.FirstName = "Nancy Constanza";
            tm.LastName = "Wautier";
            tm.Function = "Tester";
            tm.Website = url;
            tm.Picture = ResourceHelper.GetImage(typeof(AboutBox), "Resources.Nany.jpg");
            this.cv_Credits.Add(tm);

            this.cv_Credits.Navigated += (x, y) => this.timer.Stop();
            this.timer.Tick += (x, y) => this.cv_Credits.Next();
            this.timer.Interval = INTERVAL;
            this.timer.Start();

            this.FillSystemInfo();
        }

        private void AddAssemblyLvItem(string assemblyName)
        {
            try
            {
                Assembly asm = Assembly.Load(assemblyName);
                AssemblyName asmName = asm.GetName();

                ListViewItem item = new ListViewItem(asmName.Name);
                ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem(item, asmName.Version.ToString());
                item.SubItems.Add(subitem);
                lv_Components.Items.Add(item);
            }
            catch (Exception)
            {
                ListViewItem item = new ListViewItem(string.Format("'{0}' does not exist!", assemblyName));
                lv_Components.Items.Add(item);
            }
        }

        private void AddSystemLvItem(string key, string value)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.AddSystemLvItem(key, value); });
            }
            else
            {
                ListViewItem item = new ListViewItem(key);
                ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem(item, value);
                item.SubItems.Add(subitem);
                lv_System.Items.Add(item);
            }
        }

        private void FillComponentVersion()
        {
            this.AddAssemblyLvItem("Phoenix.Helpers");
            this.AddAssemblyLvItem("Phoenix.Contracts");
            this.AddAssemblyLvItem("Phoenix.Translation");
            this.AddAssemblyLvItem("Phoenix.Model.BLL");
            this.AddAssemblyLvItem("Phoenix.Model.Dal");
            this.AddAssemblyLvItem("Phoenix.Model.Core");
            this.AddAssemblyLvItem("Phoenix.View");
            this.AddAssemblyLvItem("Phoenix.Presenter");
            this.AddAssemblyLvItem("Phoenix");
            this.AddAssemblyLvItem("System.Data.SQLite");
            this.AddAssemblyLvItem("System.Windows.Forms.DataVisualization");
            this.AddAssemblyLvItem("System.Windows.Forms.Calendar");
            this.AddAssemblyLvItem("Ascend");
            this.AddAssemblyLvItem("log4net");
            this.AddAssemblyLvItem("StructureMap");
            this.AddAssemblyLvItem("WeifenLuo.WinFormsUI.Docking");
            this.AddAssemblyLvItem("PdfSharp");
            this.AddAssemblyLvItem("MigraDoc.DocumentObjectModel");
            this.AddAssemblyLvItem("MigraDoc.Rendering");
            this.AddAssemblyLvItem("NHibernate");
        }

        private void FillSystemInfo()
        {
            Thread thread = new Thread(new ThreadStart((Action)delegate
            {
                try
                {
                    this.AddSystemLvItem("Platform", Environment.OSVersion.Platform.ToString());
                    this.AddSystemLvItem("OS Version", "Windows " + Environment.OSVersion.VersionString);

                    string sp = Environment.OSVersion.ServicePack;
                    this.AddSystemLvItem("Service pack", (string.IsNullOrEmpty(sp) ? "N.A." : sp));

                    this.AddSystemLvItem("CPU Speed", string.Format("{0} Mhz", this.CPUSpeed()));

                    this.AddSystemLvItem("CLR Version", Environment.Version.ToString());

                    this.AddSystemLvItem("CommandLine", Environment.CommandLine);
                    this.AddSystemLvItem("CurrentDirectory", Environment.CurrentDirectory);

                    this.AddSystemLvItem("MachineName", Environment.MachineName);
                    this.AddSystemLvItem("UserDomainName", Environment.UserDomainName);
                    this.AddSystemLvItem("UserName", Environment.UserName);
                    this.AddSystemLvItem("ProcessorCount", Environment.ProcessorCount.ToString());
                    this.AddSystemLvItem("SystemDirectory", Environment.SystemDirectory);
                    this.AddSystemLvItem("WorkingSet", string.Format("{0:0,000} Mo", Environment.WorkingSet / 1000));

                }
                catch (Exception ex)
                {
                    this.Logger.Error(ex);
                }
            }));
            thread.Name = "System info";
            thread.Start();
        }

        private string GetLicense()
        {
            try
            {
                return ResourceHelper.GetString(Assembly.GetCallingAssembly(), "Resources.License.txt");
            }
            catch (Exception ex)
            {
                base.Logger.Error(ex);
                return "License not found!";
            }
        }

        #endregion Methods
    }
}