﻿namespace Phoenix.View.Helpers.Controls
{
    using System;
    using System.Drawing;
    using System.Windows.Forms;

    using Phoenix.Helpers.Misc;

    /// <summary>
    /// Represent a picture box with a title.
    /// </summary>
    /// <remarks>This class is sealed because it overrides the property DataBindings whose is not virtual. </remarks>
    public sealed partial class TitledImageBox : UserControl, ITitledControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TitledImageBox"/> class.
        /// </summary>
        public TitledImageBox()
        {
            InitializeComponent();

            this.linkLabel.Click += (sender, e) => this.Clicked.Raise(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the LinkLabel is clicked.
        /// </summary>
        public event EventHandler Clicked;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether the user can manually change the image of the picture box.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance can change image; otherwise, <c>false</c>.
        /// </value>
        public bool CanChangeImage
        {
            get
            {
                return this.linkLabel.Visible;
            }
            set
            {
                this.linkLabel.Visible = value;
            }
        }

        /// <summary>
        /// Gets the data bindings for the control.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// A <see cref="T:System.Windows.Forms.ControlBindingsCollection"/> that contains the <see cref="T:System.Windows.Forms.Binding"/> objects for the control.
        /// </returns>
        public new ControlBindingsCollection DataBindings
        {
            get { return this.pictureBox.DataBindings; }
        }

        /// <summary>
        /// Gets or sets the image of the component.
        /// </summary>
        /// <value>The image.</value>
        public Image Image
        {
            get { return this.pictureBox.Image; }
            set { this.pictureBox.Image = value; }
        }

        /// <summary>
        /// Gets or sets the width of the label's column.
        /// </summary>
        /// <value>The width of the label.</value>
        public float LabelWidth
        {
            get
            {
                return this.tableLayoutPanel.ColumnStyles[0].Width;
            }
            set
            {
                this.tableLayoutPanel.ColumnStyles[0].Width = value;
            }
        }

        /// <summary>
        /// Gets or sets the text of the link label.
        /// </summary>
        /// <value>The link label text.</value>
        public string LinkLabelText
        {
            get
            {
                return this.linkLabel.Text;
            }
            set
            {
                this.linkLabel.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the link label visible.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is link label visible; otherwise, <c>false</c>.
        /// </value>
        public bool LinkLabelVisible
        {
            get
            {
                return this.linkLabel.Visible;
            }
            set
            {
                this.linkLabel.Visible = value;
            }
        }

        /// <summary>
        /// Gets or sets the size mode for the picture.
        /// </summary>
        /// <value>The size mode.</value>
        public PictureBoxSizeMode SizeMode
        {
            get
            {
                return this.pictureBox.SizeMode;
            }
            set
            {
                this.pictureBox.SizeMode = value;
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get
            {
                return this.label.Text;
            }
            set
            {
                this.label.Text = value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new data binding.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="dataMember">The data member.</param>
        public void AddDataBinding(object dataSource, string dataMember)
        {
            this.pictureBox.DataBindings.Add("Image", dataSource, dataMember);
        }

        /// <summary>
        /// Clears the data binding.
        /// </summary>
        public void ClearDataBinding()
        {
            this.pictureBox.DataBindings.Clear();
        }

        #endregion Methods
    }
}