﻿namespace Phoenix.View.Input
{
    using System;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Displayer;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Translation;

    /// <summary>
    /// Provides a tool to display a Bmi entry
    /// </summary>
    public partial class BmiInput : TranslateableForm, IBmiDisplayPresenter
    {
        #region Fields

        private Usage action = Usage.Insert;
        private BMI bmi = new BMI();
        private Configuration configuration = new Configuration();
        private int lastWeight = 0;
        private BmiDisplayerModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BmiInput"/> class.
        /// </summary>             
        public BmiInput()
        {
            InitializeComponent();
            this.TranslateForm();

            this.presenter = new BmiDisplayerModel(this);
            this.Logger = LogManager.GetLogger(typeof(BmiInput));
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the BMI item to manage.
        /// </summary>
        /// <value>The selected item.</value>
        public BMI BmiToManage
        {
            get
            {

                this.bmi.Date = this.dtp_InsertDate.Value;
                this.bmi.Height = (int)this.nud_Height.Value;
                this.bmi.Weight = (double)this.nud_Weight.Value;
                return this.bmi;
            }
            set
            {
                this.bmi = value;

                this.nud_Height.Value = value.Height;
                this.nud_Weight.Value = (decimal)value.Weight;
                this.dtp_InsertDate.Value = value.Date;
            }
        }

        /// <summary>
        /// Gets or sets the height of tha patient to manage.
        /// </summary>
        /// <value>The height of the patient.</value>
        public int LastHeight
        {
            get { return this.lastWeight; }
            set
            {
                this.lastWeight = value;
                this.nud_Height.Value = this.lastWeight;
            }
        }

        /// <summary>
        /// Gets or sets the usage of this item.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get { return this.action; }
            set { this.action = value; }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        public void ClearErrors()
        {
            this.ep_error.Clear();
        }

        /// <summary>
        /// Sets the error explaining the validation fail for
        /// the specified control.
        /// </summary>
        /// <param name="failure">Indicate where the validation failed.</param>
        /// <param name="message">The message.</param>
        public void Invalidate(BmiFailedOn failure, string message)
        {
            Control control = new Control();
            string msg = string.Empty;
            switch (failure)
            {
                case BmiFailedOn.InsertDate:
                    control = this.dtp_InsertDate;
                    break;
                case BmiFailedOn.TooBigWeight:
                case BmiFailedOn.TooSmallWeight:
                    control = this.nud_Weight;
                    break;
                case BmiFailedOn.TooSmallHeight:
                case BmiFailedOn.TooBigHeight:
                    control = this.nud_Height;
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }
            this.ep_error.SetError(control, message);
        }

        private void BmiDisplayBox_Load(object sender, EventArgs e)
        {
            switch (this.action)
            {
                case Usage.Insert:
                    this.Text = "Add new bmi entry".Translate();
                    this.btn_OK.Text = "Insert".Translate();
                    break;
                case Usage.Update:
                    this.Text = "Update a bmi entry".Translate();
                    this.btn_OK.Text = "Update".Translate();
                    break;
                case Usage.Delete:
                    this.Text = "Delete a this bmi entry".Translate();
                    this.btn_OK.Text = "Delete".Translate();
                    this.nud_Height.Enabled = false;
                    this.nud_Weight.Enabled = false;
                    this.dtp_InsertDate.Enabled = false;
                    break;
                default:
                    Assert.Fails("The type {0} is not supported!", this.action);
                    break;
            }
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            if (this.ValidateData())
            {
                base.DialogResult = DialogResult.OK;
                this.bmi.Date = this.dtp_InsertDate.Value;
                this.bmi.Height = (ushort)this.nud_Height.Value;
                this.bmi.Weight = (double)this.nud_Weight.Value;
                this.Close();
            }
        }

        private void NumericUpDown_Enter(object sender, EventArgs e)
        {
            NumericUpDown control = sender as NumericUpDown;
            Assert.IsTrue((control != null)
                , string.Format("The expected type is 'NumericUpDown' but is '{0}'", sender.GetType().Name));
            control.Select(0, control.Text.Length);
        }

        private bool ValidateData()
        {
            return this.presenter.ValidateData();
        }

        #endregion Methods

        #region Nested Types

        /// <summary>
        /// Provides configuration data for the display.
        /// </summary>
        public class Configuration
        {
            #region Constructors

            /// <summary>
            /// Initializes a new instance of the <see cref="Configuration"/> class.
            /// Gives default values.
            /// </summary>
            public Configuration()
            {
                this.MaxHeight = 272;
                this.MaxWeight = 560;
            }

            #endregion Constructors

            #region Properties

            /// <summary>
            /// Gets or sets the max height.
            /// </summary>
            /// <value>The height of the max.</value>
            public ushort MaxHeight
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the max weight.
            /// </summary>
            /// <value>The max weight.</value>
            public double MaxWeight
            {
                get;
                set;
            }

            #endregion Properties
        }

        #endregion Nested Types
    }
}