﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Presenter.Interfaces.Searcher;

    /// <summary>
    /// Provide a form to manage the relations of a patient
    /// </summary>
    public partial class RelationInput : TranslateableForm, IRelationInputPresenter
    {
        #region Fields

        private DialogResult dialogResult = DialogResult.Cancel;
        private RelationModel presenter;
        private Relation selectedRelation;
        private Usage usage;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelationInput"/> class.
        /// </summary>
        public RelationInput()
        {
            InitializeComponent();
            this.presenter = new RelationModel(this);
            this.Usage = Usage.Insert;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Patient Patient
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the relations.
        /// </summary>
        /// <value>The relations.</value>
        public List<RelationType> Relations
        {
            get
            {
                return this.cb_FamilyType.DataSource as List<RelationType>;
            }
            set
            {
                this.cb_FamilyType.DataSource = null;
                this.cb_FamilyType.DataSource = value;
            }
        }

        /// <summary>
        /// Gets or sets the selected relation.
        /// </summary>
        /// <value>The selected relation.</value>
        public Relation SelectedRelation
        {
            get { return this.selectedRelation; }
            set
            {
                if (value == null) return;

                this.selectedRelation = value;
                this.Patient = value.UpsideDown.Member;
                this.btn_Parent.Tag = value.Member;
                this.btn_Parent.Text = this.Patient.ToString();
                this.cb_FamilyType.SelectedItem = value.Type;
            }
        }

        /// <summary>
        /// Gets or sets the usage.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get
            {
                return this.usage;
            }
            set
            {
                this.usage = value;
                switch (value)
                {
                    case Usage.Insert:
                        this.btn_Parent.Enabled = true;
                        this.btn_OK.Enabled = false;
                        break;
                    case Usage.Update:
                        this.btn_Parent.Enabled = false;
                        this.btn_OK.Enabled = true;
                        break;
                    case Usage.Delete:
                    case Usage.Display:
                    default:
                        Assert.FailsOnEnumeration(value);
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the error.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="failure">The failure.</param>
        /// <param name="message">The message.</param>
        public void SetError(RelationFailedOn failure, string message)
        {
            Control control = new Control();

            switch (failure)
            {
                case RelationFailedOn.NoParent:
                    control = this.btn_OK;
                    break;
                case RelationFailedOn.NoRelationType:
                    control = this.cb_FamilyType;
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }

            this.ep_Error.SetError(control, message);
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            this.FillSelectedRelation();

            if (this.presenter.ValidateData())
            {
                this.dialogResult = DialogResult.OK;
                this.Close();
            }
        }

        private void btn_Parent_Click(object sender, EventArgs e)
        {
            IPatientSearcher box = GuiManager.Get<IPatientSearcher>();
            System.Windows.Forms.DialogResult dr = GuiManager.ShowDialog(box);
            if (dr != System.Windows.Forms.DialogResult.OK) return;

            this.btn_Parent.Text = box.SelectedItem.ToString();
            this.btn_Parent.Tag = box.SelectedItem;
            this.btn_OK.Enabled = true;
        }

        private void FillSelectedRelation()
        {
            #region Temporary relation
            Relation tempRelation = (this.SelectedRelation == null)
                ? new Relation()
                : this.SelectedRelation;

            tempRelation.Type = this.cb_FamilyType.SelectedItem as RelationType;
            tempRelation.Member = this.btn_Parent.Tag as Patient;
            #endregion

            #region UpsideDown relation
            Relation upsideDown = (tempRelation.UpsideDown == null)
                ? new Relation()
                : this.selectedRelation.UpsideDown;

            upsideDown.Type = tempRelation.Type.UpsideDown;
            upsideDown.Member = this.Patient;
            upsideDown.UpsideDown = tempRelation;
            #endregion

            tempRelation.UpsideDown = upsideDown;
            this.SelectedRelation = tempRelation;
        }

        private void RelationManagerBox_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.DialogResult = dialogResult;
        }

        private void RelationManagerBox_Load(object sender, EventArgs e)
        {
            this.presenter.RefreshData();
            if (this.SelectedRelation != null
                && this.SelectedRelation.Type != null
                && !this.SelectedRelation.Type.IsEmpty)
            {
                this.cb_FamilyType.SelectedItem = this.selectedRelation.Type;
            }
        }

        #endregion Methods
    }
}