﻿namespace Phoenix.Presenter.Search.Criterion
{
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    /// <summary>
    /// Displays a generic activable criterion for any search.
    /// </summary>
    public partial class CriterionControl : UserControl, ICriterionControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CriterionControl"/> class.
        /// </summary>
        public CriterionControl()
        {
            InitializeComponent();
            this.IsActivated = true;
            this.Locked = false;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the name of the criterion.
        /// </summary>
        /// <value></value>
        /// <number>The name of the criterion.</number>
        public string CriterionName
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name of the criterion.
        /// </summary>
        /// <number>The name of the criterion.</number>
        [Category("Setup")]
        [Description("Gets or sets the name of the criterion.")]
        public string CriterionText
        {
            get
            {
                return this.lbl_label.Text;
            }
            set
            {
                this.lbl_label.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets a number indicating whether this criterion is activated.        
        /// </summary>
        /// <number>
        /// 	<c>true</c> if this criterion is activated then it'll be part
        /// 	of the criterion on the next search; otherwise, <c>false</c>.
        /// </number>
        [Category("Setup")]
        [Description("Gets or sets a value indicating whether this criterion is activated. ")]
        public bool IsActivated
        {
            get
            {
                return this.chb_checkBox.Checked;
            }
            set
            {
                this.chb_checkBox.Checked = this.pnl_panel.Enabled = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is control.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is control; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IsControl
        {
            get { return ((this as Control) != null); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="ICriterionControl"/> is locked.
        /// That's, you can't unckeck this criterion.
        /// </summary>
        /// <value><c>true</c> if locked; otherwise, <c>false</c>.</value>
        public bool Locked
        {
            get { return !this.chb_checkBox.Visible; }
            set { this.chb_checkBox.Visible = !value; }
        }

        /// <summary>
        /// Gets or sets the name of the criterion.
        /// </summary>
        /// <value></value>
        /// <returns>The text associated with this control.</returns>
        public override string Text
        {
            get
            {
                return this.CriterionText;
            }
            set
            {
                this.CriterionText = value;
            }
        }

        /// <summary>
        /// Gets the criterion's number.
        /// </summary>
        /// <number>The criterion's number.</number>
        [Browsable(false)]
        public virtual object Value
        {
            get { return null; }
        }

        /// <summary>
        /// Gets the check box.
        /// </summary>
        /// <value>The check box.</value>
        protected CheckBox CheckBox
        {
            get { return this.chb_checkBox; }
        }

        /// <summary>
        /// Gets the panel that receive the criterion.
        /// </summary>
        /// <value>The panel.</value>
        protected Panel Panel
        {
            get { return this.pnl_panel; }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the error provider of messages.
        /// </summary>
        public void ClearError()
        {
            this.ep_error.Clear();
        }

        /// <summary>
        /// Gives the focus to the input control. 
        /// For instance, it could be the TextBox or the NumericUpDown
        /// </summary>
        public void SelectInput()
        {
            if (pnl_panel.Controls.Count > 0)
                this.pnl_panel.Controls[0].Select();
        }

        /// <summary>
        /// Display the specified messgae into the error provider.
        /// </summary>
        /// <param name="message">The message.</param>
        public void SetError(string message)
        {
            this.ep_error.SetError(this.lbl_ErrorProvider, message);
        }

        /// <summary>
        /// Checks the values the user entered and provides
        /// warning message if values are not correct.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the check succeed. Otherwise <c>False</c>.
        /// </returns>
        public virtual bool ValidateCriterions()
        {
            return true;
        }

        /// <summary>
        /// Adds the specified control in criterion control
        /// as the input control.
        /// </summary>
        /// <param name="item">The item.</param>
        protected void AddInCriterionControl(Control item)
        {
            item.Dock = DockStyle.Fill;
            this.pnl_panel.Controls.Add(item);
        }

        /// <summary>
        /// Clears the value of the criterion.
        /// </summary>
        protected virtual void ClearValue()
        {
        }

        private void chb_checkBox_CheckedChanged(object sender, EventArgs e)
        {
            this.pnl_panel.Enabled = this.chb_checkBox.Checked;
            if (!this.chb_checkBox.Checked)
            {
                this.ClearError();
                this.ClearValue();
            }
            else
            {
                this.SelectInput();
            }
        }

        #endregion Methods
    }
}